@@ -211,11 +211,7 @@ QAudioInput::~QAudioInput()
      If a problem occurs during this process the error() is set to QAudio::OpenError,
      state() is set to QAudio::StoppedState and stateChanged() signal is emitted.
 
-    In either case, the stateChanged() signal may be emitted either synchronously
-    during execution of the start() function or asynchronously after start() has
-    returned to the caller.
-
-     \sa {Symbian Platform Security Requirements}
+     \l{QAudioInput#Symbian Platform Security Requirements}
 
      \sa QIODevice
 */
@@ -237,11 +233,7 @@ void QAudioInput::start(QIODevice* device)
     If a problem occurs during this process the error() is set to QAudio::OpenError,
     state() is set to QAudio::StoppedState and stateChanged() signal is emitted.
 
-    In either case, the stateChanged() signal may be emitted either synchronously
-    during execution of the start() function or asynchronously after start() has
-    returned to the caller.
-
-    \sa {Symbian Platform Security Requirements}
+    \l{QAudioInput#Symbian Platform Security Requirements}
 
     \sa QIODevice
 */
@@ -286,8 +278,6 @@ void QAudioInput::reset()
 
     Sets error() to QAudio::NoError, state() to QAudio::SuspendedState and
     emit stateChanged() signal.
-
-    Note: signal will always be emitted during execution of the resume() function.
 */
 
 void QAudioInput::suspend()
@@ -310,7 +300,7 @@ void QAudioInput::resume()
 }
 
 /*!
-    Sets the audio buffer size to \a value bytes.
+    Sets the audio buffer size to \a value milliseconds.
 
     Note: This function can be called anytime before start(), calls to this
     are ignored after start(). It should not be assumed that the buffer size
@@ -325,7 +315,7 @@ void QAudioInput::setBufferSize(int value)
 }
 
 /*!
-    Returns the audio buffer size in bytes.
+    Returns the audio buffer size in milliseconds.
 
     If called before start(), returns platform default value.
     If called before start() but setBufferSize() was called prior, returns value set by setBufferSize().
